/*********************************************************************
 * 
 *    Filename:  output.c
 *    Author:    Ryan Shillington <shillington@writeme.com>
 *    Description: This reads the key file, and outputs the right key for
 *                  the IR code that is read in.
 *  
 *     This program is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU General Public License as
 *     published by the Free Software Foundation; either version 2 of
 *     the License, or (at your option) any later version.
 *
 ********************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include "output.h"
#include <string.h>

#define CONFIGFILENAME "default.cfg"

//* This is true when the CAPS lock of SHIFT key is pressed
int CAPS_LOCK;

Key *keys;
int numKeys;

int caps = 0;
char *configFileName;

int main(int argc, char **argv)
{
  int i, len;
  char data[64];
  int irportFileID;

  //* Deal with the command line arguments;
  if (argc == 1) {
    configFileName = CONFIGFILENAME;
  } else if (argc==2) {
    configFileName = argv[1];
  } else {
    fprintf(stderr, "USAGE: output <device_config.cfg>\n\n");
    exit(1);
  }

  //* Get the Keys
  parseKeys();
  DEBUG(5, "Number of Keys = %d\n", numKeys);
  
  //* Open the IrPort
  if ((irportFileID = open("/dev/irport", O_RDWR)) == -1)
  {
    DEBUG(0, "Error opening /dev/irport.\n%s\n", strerror(errno));
    DEBUG(0, "Are you sure that you installed the ir-remote modules?\n\n");
    exit(1);
  }
  
  while(!((keys[i].key < 0) && (strcmp(keys[i].keyChar, "exit") == 0)))
  {
    //* Read the next key
    memset(data, 0, 64);
    len = read(irportFileID, data, 64);

    //* Try to find it in the database
    for(i = 0; 
        ((i < numKeys) && 
         (diff(data, len, keys[i].code, keys[i].codeLen, 1))); 
        i++) ;
    
    //* Print a character, if we recognized the key
    if(i < numKeys) {
      if(keys[i].key > 0) {
        if(caps)
        { 
          if ((keys[i].key >= 'a') && (keys[i].key <= 'z'))
          {
            DEBUG(1, "%c", (char)(keys[i].key - 32));
          } else {
            DEBUG(1, "%c", (char)keys[i].key);
          }
        } else {
          DEBUG(1, "%c", (char)keys[i].key);
        }
      } else {
        handleSpecialChar(i);
      }
    }

    DEBUG(5, "Code = ");
    for(i = 0; i < len; i++)
      DEBUG(5, "%x ", data[i]);
    DEBUG(5, "\n");

    fflush(stdout);
  }
}

void handleSpecialChar(int i)
{
  switch(keys[i].key)
  {
    case CAPS: caps= ~caps;
               break;
    default: DEBUG(1, "%s\n", keys[i].keyChar);
             break;
  }
}
int diff(char *str1, int len1,  char *str2, int len2, int maxdiff)
{
  int i, maxlen;
  int diff = 0;

  //* match the fewest number of characters
  maxlen = (len1 > len2) ? len2 : len1;
  
  for(i = 0; (i < maxlen) && (diff <= maxdiff); i++)
  {
    DEBUG(5, "%x %x\n", str1[i], str2[i]);
    if(str1[i] != str2[i]) diff++;
  }
  
  if(diff > maxdiff)
  {
    DEBUG(5, "return 1\n");
    return 1;
  } else { 
    DEBUG(5, "return 0\n");
    return 0;
  }
}

void parseKeys()
{
  struct stat Stat;
  int configFileID;
  int curr, find, i;
  char *buff, *temp;
  
  numKeys = 0;

  //* 1 - Open the configuration file
  if((configFileID = open(configFileName, O_RDONLY)) == -1)
  {
    DEBUG(0, "Error opening configuration file (%s).\n%s\n", 
        configFileName, strerror(errno));
    exit(1);
  }
  
  //* 2 - Read the configuration file
  fstat(configFileID, &Stat);
  buff = (char *) malloc(Stat.st_size+1);
  read(configFileID, buff, Stat.st_size);
  buff[Stat.st_size] = '\0';
  DEBUG(5, "File Size = %d\n", Stat.st_size);
  
  //* 3 - Count the number of keys
  for(i = 0; i < Stat.st_size; i++)
    if (buff[i] == '\n') 
    {
      numKeys++;
      i++;
    }
  
  //* 4 - Allocate memory for the Keys
  keys = (Key *) malloc(sizeof(Key) * numKeys);
  
  //* 5 - Parse each and every key
  find = 0;
  for(i = 0; i < numKeys; i++)
  {
    //* 5.1 - Find the key
    temp = buff+find;
    while(buff[find] != '\t') find++;
    buff[find] = '\0';
    find++;
    
    //* 5.2 - Convert the Key
    keys[i].key = convertKey(temp);
    if(keys[i].key < 0)
    {
      keys[i].keyChar = (char *) malloc(strlen(temp)+1);
      strcpy(keys[i].keyChar, temp);
    }

    //* 5.3 - Find the code (and it's length) for this key
    temp = buff+find;
    curr = 0;
    while((buff[find] != '\n') || (buff[find+1] != '\n'))
      keys[i].code[curr++] = buff[find++];
    keys[i].codeLen = (buff+find) - temp;
    find+=2;
    
    DEBUG(5, "#%d - Key = %d, Code = %x\n", i, keys[i].key, keys[i].code[0]);
  }

  //* Delete the memory used
  free(buff);
}
  
int convertKey(char *temp)
{
  //* Look to see if this is a special character (ie. ESC, Caps Lock, etc.)
  if(strlen(temp) > 1)
  {
    if(strcmp(temp, "CAPS") == 0)
    {
      return CAPS;
    } else if(strcmp(temp, "ENTER") == 0)
    {
      return '\n';
    } else if(strcmp(temp, "SPACE") == 0)
    {
      return ' ';
    } else if(strcmp(temp, "TAB") == 0)
    {
      return '\t';
    } else if(strcmp(temp, "BS") == 0)
    {
      return '\b';
    } else if(strcmp(temp, "WINDOWS") == 0)
    {
      return 3;
    } else {
      //* Not defined?
      return -1;
    }
  } else {
    //* Return the ascii character code
    return (int) temp[0];
  }
}
